@@ -51,6 +51,9 @@
 #define AFSOP_SET_RXMAXMTU       40     /* set rx_MyMaxSendSize,rx_maxReceiveSizeUser,rx_maxReceiveSize */
 #define AFSOP_BKG_HANDLER        41     /* userspace-capable Bkg daemon */
 #define AFSOP_SET_RXMAXFRAGS     43     /* set rxi_nSendFrags, rxi_nRecvFrags */
+#define AFSOP_SET_RMTSYS_FLAG    44     /* set flag if rmtsys is enabled */
+
+#define AFSOP_SET_INUMCALC       46     /* set inode number calculation method */
 
 /* The range 20-30 is reserved for AFS system offsets in the afs_syscall */
 #define	AFSCALL_PIOCTL		20
@@ -174,6 +177,13 @@ struct afs_cacheParams {
     afs_int32 dynamic_vcaches;
 };
 
+/* Supported values for AFSOP_SET_INUMCALC. */
+enum {
+    AFS_INUMCALC_COMPAT = 0,
+    AFS_INUMCALC_MD5 = 1
+};
+
+
 /*
  * Note that the AFS_*ALLOCSIZ values should be multiples of sizeof(void*) to
  * accomodate pointer alignment.
