@@ -82,6 +82,9 @@
 #define SYS_NAME_ID_x86_darwin_150       529
 #define SYS_NAME_ID_amd64_darwin_150     530
 #define SYS_NAME_ID_arm_darwin_150       531
+#define SYS_NAME_ID_x86_darwin_160       532
+#define SYS_NAME_ID_amd64_darwin_160     533
+#define SYS_NAME_ID_arm_darwin_160       534
 
 #define SYS_NAME_ID_next_mach20		 601
 #define SYS_NAME_ID_next_mach30		 602
